"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|
"""

# Starting and global variables

window_position = "+300+200"

PROPORTIONAL_FONT_FAMILY = ('Arial',"MS", "Sans", "Serif")
MONOSPACE_FONT_FAMILY = "Arial"

# PROPORTIONAL_FONT_SIZE = 10
PROPORTIONAL_FONT_SIZE = 30
# a little smaller, because it is more legible at a smaller size
# MONOSPACE_FONT_SIZE = 9
MONOSPACE_FONT_SIZE = 50
# TEXT_ENTRY_FONT_SIZE = 12  # a little larger makes it easier to see
TEXT_ENTRY_FONT_SIZE = 30  # a little larger makes it easier to see


STANDARD_SELECTION_EVENTS = ["Return", "Button-1", "space"]

# prop_font_line_length = 62
# fixw_font_line_length = 80
# num_lines_displayed = 50
# default_hpad_in_chars = 2


prop_font_line_length = 50
fixw_font_line_length = 80
num_lines_displayed = 50
default_hpad_in_chars = 3
